from database import db, app, cache
from flask_covid19.blueprints.app_all.all_config import BlueprintConfig
from flask_covid19.blueprints.app_web.web_model_factory import BlueprintDateReportedFactory
from flask_covid19.blueprints.data_who.who_model import WhoCountryRegion, WhoDateReported, WhoCountry, WhoData
from flask_covid19.blueprints.data_who.who_model_import import WhoImport


class WhoServiceUpdateBase:
    def __init__(self, database, config: BlueprintConfig):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" WhoServiceUpdateBase [init]")
        app.logger.debug("------------------------------------------------------------")
        self.__database = database
        self.cfg = config
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" WhoServiceUpdateBase [ready]")


class WhoServiceUpdateFull(WhoServiceUpdateBase):

    def __full_update_date_reported(self):
        app.logger.info(" WhoServiceUpdateFull.__full_update_date_reported [begin]")
        app.logger.info("------------------------------------------------------------")
        WhoDateReported.remove_all()
        with app.app_context():
            cache.clear()
        log_lines = []
        i = 0
        # myresultarray = []
        # myresultset = WhoImport.get_all_datum()
        # for my_datum_item in myresultset:
        #    my_datum = my_datum_item.datum
        #    if not my_datum in myresultarray:
        #        myresultarray.append(my_datum)
        # for a in myresultset:
        #    app.logger.info(str(a))
        # for b in WhoImport.get_dates_reported_as_string_array():
        #    app.logger.info(str(b))
        for s_date_reported in WhoImport.get_dates_reported_as_string_array():
            i += 1
            o = BlueprintDateReportedFactory.create_new_object_for_who(my_date_rep=s_date_reported)
            db.session.add(o)
            a = str(o)
            b = str(i)
            output = " full update WHO date_reported ... " + b + " rows ... (" + a + ")"
            log_lines.append(output)
        db.session.commit()
        for log_line in log_lines:
            app.logger.info(log_line)
        app.logger.info("")
        app.logger.info(" WhoServiceUpdateFull.__full_update_date_reported [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def __full_update_region(self):
        app.logger.info(" WhoServiceUpdateFull.__full_update_region [begin]")
        app.logger.info("------------------------------------------------------------")
        WhoCountryRegion.remove_all()
        log_lines = []
        i = 0
        for i_who_region, in WhoImport.get_regions():
            i += 1
            o = WhoCountryRegion(
                location_group=i_who_region,
                processed_update=False,
                processed_full_update=False,
            )
            db.session.add(o)
            a = str(o)
            b = str(i)
            output = " full update WHO region ... " + b + " rows ... (" + a + ")"
            log_lines.append(output)
        db.session.commit()
        for log_line in log_lines:
            app.logger.info(log_line)
        app.logger.info("")
        app.logger.info(" WhoServiceUpdateFull.__full_update_region [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def __full_update_country(self):
        app.logger.info(" WhoServiceUpdateFull.__full_update_country [begin]")
        app.logger.info("------------------------------------------------------------")
        WhoCountry.remove_all()
        self.__full_update_region()
        with app.app_context():
            cache.clear()
        result = WhoImport.countries()
        log_lines = []
        i = 0
        for result_item in result:
            i += 1
            i_country_code = result_item.countries.country_code
            i_country = result_item.countries.country
            i_who_region = result_item.countries.who_region
            my_region = WhoCountryRegion.find_by_location_group(i_who_region)
            o = WhoCountry(
                location=i_country,
                location_code=i_country_code,
                location_group=my_region,
                processed_update=False,
                processed_full_update=False)
            db.session.add(o)
            a = str(o)
            b = str(i)
            output = " full update WHO country ... " + b + " rows ... (" + a + ")"
            log_lines.append(output)
        db.session.commit()
        for log_line in log_lines:
            app.logger.info(log_line)
        app.logger.info("")
        app.logger.info(" WhoServiceUpdateFull.__full_update_country [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def __full_update_data(self):
        app.logger.info(" WhoServiceUpdateFull.__full_update_data [begin]")
        app.logger.info("------------------------------------------------------------")
        app.logger.info(" WhoData.remove_all() [begin]")
        WhoData.remove_all()
        with app.app_context():
            cache.clear()
        app.logger.info(" WhoData.remove_all() [done]")
        new_dates_reported_as_string_array = WhoImport.get_dates_reported_as_string_array()
        new_dates_reported_from_import = []
        for my_date_reported_as_string in new_dates_reported_as_string_array:
            # app.logger.info(" my_date: " + my_date_reported_as_string)
            my_date = WhoDateReported.get_by_date_reported(my_date_reported_as_string)
            # app.logger.info(" my_date: " + str(my_date))
            new_dates_reported_from_import.append(my_date)
        # app.logger.info("------------------------------------------------------------")
        i = 0
        d = 0
        k = 0
        for my_date in new_dates_reported_from_import:
            # app.logger.info(" my_date: " + str(my_date))
            for my_who_import in WhoImport.find_by_datum(my_date.datum):
                # app.logger.info(" my_who_import: " + str(my_who_import))
                my_country = WhoCountry.find_by_location_code_and_location(
                    location_code=my_who_import.country_code,
                    location=my_who_import.country
                )
                # app.logger.info(" my_country: " + str(my_country))
                o = WhoData(
                    cases_new=int(my_who_import.new_cases),
                    cases_cumulative=int(my_who_import.cumulative_cases),
                    deaths_new=int(my_who_import.new_deaths),
                    deaths_cumulative=int(my_who_import.cumulative_deaths),
                    date_reported=my_date,
                    location=my_country,
                    processed_update=False,
                    processed_full_update=False
                )
                db.session.add(o)
                my_who_import.row_imported = True
                db.session.add(my_who_import)
                i += 1
                k += 1
            d += 1
            if d % 7 == 0:
                db.session.commit()
                s1 = str(i)
                s2 = str(my_date)
                s3 = str(k)
                app.logger.info(" full update WHO data ... " + s1 + " rows ... " + s2 + " (" + s3 + ")")
                k = 0
        db.session.commit()
        app.logger.info(" full update WHO data:  "+str(i)+" total rows")
        app.logger.info(" WhoServiceUpdateFull.__full_update_data [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def full_update_dimension_tables(self):
        app.logger.info(" WhoServiceUpdateFull.full_update_dimension_tables [begin]")
        app.logger.info("------------------------------------------------------------")
        self.__full_update_date_reported()
        self.__full_update_country()
        app.logger.info(" WhoServiceUpdateFull.full_update_dimension_tables [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def full_update_fact_table(self):
        app.logger.info(" WhoServiceUpdateFull.full_update_fact_table [begin]")
        app.logger.info("------------------------------------------------------------")
        self.__full_update_data()
        app.logger.info(" WhoServiceUpdateFull.full_update_fact_table [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def full_update_star_schema(self):
        app.logger.info(" WhoServiceUpdateFull.full_update_star_schema [begin]")
        app.logger.info("------------------------------------------------------------")
        WhoData.remove_all()
        self.__full_update_date_reported()
        self.__full_update_country()
        self.__full_update_data()
        app.logger.info(" WhoServiceUpdateFull.full_update_star_schema [done]")
        app.logger.info("------------------------------------------------------------")
        return None


class WhoServiceUpdate(WhoServiceUpdateBase):

    def __who_import_get_new_dates_as_array(self):
        new_dates_reported_from_import = []
        list_datum_of_all_who_data = WhoData.get_datum_of_all_data()
        for item in WhoImport.get_datum_of_all_who_import():
            if not item in list_datum_of_all_who_data:
                new_dates_reported_from_import.append(item)
        return new_dates_reported_from_import

    def __update_date_reported(self):
        app.logger.info(" WhoServiceUpdate.__update_date_reported [begin]")
        app.logger.info("------------------------------------------------------------")
        i = 0
        for i_date_reported in self.__who_import_get_new_dates_as_array():
            i += 1
            output = " [ " + str(i) + " ] " + i_date_reported
            c = WhoDateReported.find_by_date_reported(i_date_reported)
            if c is None:
                o = WhoDateReported.create_new_object_factory(my_date_rep=i_date_reported)
                db.session.add(o)
                db.session.commit()
                output += "   added"
            else:
                output += "   NOT added "+str(c.id)
            app.logger.info(output)
        app.logger.info("")
        app.logger.info(" WhoServiceUpdate.__update_date_reported [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def __update_region(self):
        app.logger.info(" WhoServiceUpdate.__update_region [begin]")
        app.logger.info("------------------------------------------------------------")
        i = 0
        for i_who_region, in WhoImport.get_regions():
            i += 1
            output = " [ " + str(i) + " ] " + i_who_region
            c = WhoCountryRegion.find_by_region(i_who_region)
            if c is None:
                o = WhoCountryRegion(
                    region=i_who_region,
                    processed_update=False,
                    processed_full_update=False
                )
                db.session.add(o)
                db.session.commit()
                output += "   added"
            else:
                output += "   NOT added ( " + str(c.id) + " ) "
            app.logger.info(output)
        app.logger.info("")
        app.logger.info(" WhoServiceUpdate.__update_region [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def __update_country(self):
        app.logger.info(" WhoServiceUpdate.__update_country [begin]")
        app.logger.info("------------------------------------------------------------")
        self.__update_region()
        result = WhoImport.countries()
        i = 0
        for result_item in result:
            i += 1
            i_country_code = result_item.countries.country_code
            i_country = result_item.countries.country
            i_who_region = result_item.countries.who_region
            output = " [ " + str(i) + " ] " + i_country_code + " | " + i_country + " | " + i_who_region + " | "
            my_region = WhoCountryRegion.find_by_region(i_who_region)
            my_country = WhoCountry.find_by_country_code_and_country_and_who_region_id(
                i_country_code, i_country, my_region
            )
            if my_country is None:
                o = WhoCountry(
                    country=i_country,
                    country_code=i_country_code,
                    region=my_region,
                    processed_update=False,
                    processed_full_update=False
                )
                db.session.add(o)
                db.session.commit()
                output2 = "   added "
            else:
                output2 = "   NOT added ( " + str(my_country.id) + " ) "
            output += i_country_code + output2
            app.logger.info(output)
        db.session.commit()
        app.logger.info("")
        app.logger.info(" WhoServiceUpdate.__update_country [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def __update_data(self):
        app.logger.info(" WhoServiceUpdate.__update_data [begin]")
        app.logger.info("------------------------------------------------------------")
        new_dates_reported_from_import = self.__who_import_get_new_dates_as_array()
        i = 0
        d = 0
        k = 0
        for my_date_reported in new_dates_reported_from_import:
            app.logger.info(my_date_reported)
            my_date = WhoDateReported.find_by_date_reported(my_date_reported)
            if my_date is None:
                myday = WhoDateReported.create_new_object_factory(my_date_reported)
                db.session.add(myday)
                db.session.commit()
            my_date = WhoDateReported.get_by_date_reported(my_date_reported)
            for result_item in WhoImport.get_for_one_day(my_date_reported):
                if result_item.country_code == "":
                    my_country = WhoCountry.get_by_country(result_item.country)
                else:
                    my_country = WhoCountry.get_by_country_code(result_item.country_code)
                o = WhoData(
                    cases_new=int(result_item.new_cases),
                    cases_cumulative=int(result_item.cumulative_cases),
                    deaths_new=int(result_item.new_deaths),
                    deaths_cumulative=int(result_item.cumulative_deaths),
                    date_reported=my_date,
                    country=my_country,
                    processed_update=False,
                    processed_full_update=False
                )
                db.session.add(o)
                i += 1
                k += 1
            d += 1
            if d % 7 == 0:
                db.session.commit()
                app.logger.info(" update WHO  " + str(my_date) + " ... " + str(i) + " rows ( " + str(k) + " )")
                k = 0
        db.session.commit()
        app.logger.info(" update WHO incremental :  "+str(i)+" rows total")
        app.logger.info(" WhoServiceUpdate.__update_data [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def update_dimension_tables(self):
        app.logger.info(" update_dimension_tables_only [begin]")
        app.logger.info("------------------------------------------------------------")
        self.__update_date_reported()
        self.__update_country()
        app.logger.info(" update_dimension_tables_only [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def update_fact_table(self):
        app.logger.info(" update_fact_table_only [begin]")
        app.logger.info("------------------------------------------------------------")
        self.__update_data()
        app.logger.info(" update_fact_table_only [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def update_star_schema(self):
        app.logger.info(" update_star_schema [begin]")
        app.logger.info("------------------------------------------------------------")
        self.update_dimension_tables()
        self.update_fact_table()
        app.logger.info(" update_star_schema [done]")
        app.logger.info("------------------------------------------------------------")
        return self
